PropertyValue

Enum PropertyValue 

pub enum PropertyValue {
Show 21 variants Float(f32), FloatArray(Vec<f32>), Int(i32), IntArray(Vec<i32>), UInt(u32), UIntArray(Vec<u32>), Vector2(Vector2<f32>), Vector2Array(Vec<Vector2<f32>>), Vector3(Vector3<f32>), Vector3Array(Vec<Vector3<f32>>), Vector4(Vector4<f32>), Vector4Array(Vec<Vector4<f32>>), Matrix2(Matrix2<f32>), Matrix2Array(Vec<Matrix2<f32>>), Matrix3(Matrix3<f32>), Matrix3Array(Vec<Matrix3<f32>>), Matrix4(Matrix4<f32>), Matrix4Array(Vec<Matrix4<f32>>), Bool(bool), Color(Color), Sampler { value: Option<Texture>, fallback: SamplerFallback, },
}
A value of a property that will be used for rendering with a shader.

Limitations

There is a limited set of possible types that can be passed to a shader, most of them are just simple data types.

Variants



Float(f32)

Real number.



FloatArray(Vec<f32>)

Real number array.



Int(i32)

Integer number.



IntArray(Vec<i32>)

Integer number array.



UInt(u32)

Natural number.



UIntArray(Vec<u32>)

Natural number array.



Vector2(Vector2<f32>)

Two-dimensional vector.



Vector2Array(Vec<Vector2<f32>>)

Two-dimensional vector array.



Vector3(Vector3<f32>)

Three-dimensional vector.



Vector3Array(Vec<Vector3<f32>>)

Three-dimensional vector array.



Vector4(Vector4<f32>)

Four-dimensional vector.



Vector4Array(Vec<Vector4<f32>>)

Four-dimensional vector array.



Matrix2(Matrix2<f32>)

2x2 Matrix.



Matrix2Array(Vec<Matrix2<f32>>)

2x2 Matrix array.



Matrix3(Matrix3<f32>)

3x3 Matrix.



Matrix3Array(Vec<Matrix3<f32>>)

3x3 Matrix array.



Matrix4(Matrix4<f32>)

4x4 Matrix.



Matrix4Array(Vec<Matrix4<f32>>)

4x4 Matrix array.



Bool(bool)

Boolean value.



Color(Color)

An sRGB color.

Conversion

The colors you see on your monitor are in sRGB color space, this is fine for simple cases of rendering, but not for complex things like lighting. Such things require color to be linear. Value of this variant will be automatically converted to linear color space before it passed to shader.



Sampler

A texture with fallback option.

Fallback

Sometimes you don’t want to set a value to a sampler, or you even don’t have the appropriate one. There is fallback value that helps you with such situations, it defines a values that will be fetched from a sampler when there is no texture.

For example, standard shader has a lot of samplers defined: diffuse, normal, height, emission, mask, metallic, roughness, etc. In some situations you may not have all the textures, you have only diffuse texture, to keep rendering correct, each other property has appropriate fallback value. Normal sampler - a normal vector pointing up (+Y), height - zero, emission - zero, etc.

Fallback value is also helpful to catch missing textures, you’ll definitely know the texture is missing by very specific value in the fallback texture.

Fields

value: Option<Texture>

Actual value of the sampler. Could be None, in this case fallback will be used.

fallback: SamplerFallback

Sampler fallback value.

Implementations

Source

impl PropertyValue

pub fn as_float(&self) -> Option<f32>

Tries to unwrap property value as float.

pub fn as_float_array(&self) -> Option<&[f32]>

Tries to unwrap property value as float array.

pub fn as_int(&self) -> Option<i32>

Tries to unwrap property value as integer.

pub fn as_int_array(&self) -> Option<&[i32]>

Tries to unwrap property value as integer array.

pub fn as_uint(&self) -> Option<u32>

Tries to unwrap property value as unsigned integer.

pub fn as_uint_array(&self) -> Option<&[u32]>

Tries to unwrap property value as unsigned integer array.

pub fn as_bool(&self) -> Option<bool>

Tries to unwrap property value as boolean.

pub fn as_color(&self) -> Option<Color>

Tries to unwrap property value as color.

pub fn as_vector2(&self) -> Option<Vector2<f32>>

Tries to unwrap property value as two-dimensional vector.

pub fn as_vector2_array(&self) -> Option<&[Vector2<f32>]>

Tries to unwrap property value as two-dimensional vector array.

pub fn as_vector3(&self) -> Option<Vector3<f32>>

Tries to unwrap property value as three-dimensional vector.

pub fn as_vector3_array(&self) -> Option<&[Vector3<f32>]>

Tries to unwrap property value as three-dimensional vector array.

pub fn as_vector4(&self) -> Option<Vector4<f32>>

Tries to unwrap property value as four-dimensional vector.

pub fn as_vector4_array(&self) -> Option<&[Vector4<f32>]>

Tries to unwrap property value as four-dimensional vector array.

pub fn as_matrix2(&self) -> Option<Matrix2<f32>>

Tries to unwrap property value as 2x2 matrix.

pub fn as_matrix2_array(&self) -> Option<&[Matrix2<f32>]>

Tries to unwrap property value as 2x2 matrix array.

pub fn as_matrix3(&self) -> Option<Matrix3<f32>>

Tries to unwrap property value as 3x3 matrix.

pub fn as_matrix3_array(&self) -> Option<&[Matrix3<f32>]>

Tries to unwrap property value as 3x3 matrix array.

pub fn as_matrix4(&self) -> Option<Matrix4<f32>>

Tries to unwrap property value as 4x4 matrix.

pub fn as_matrix4_array(&self) -> Option<&[Matrix4<f32>]>

Tries to unwrap property value as 4x4 matrix array.

pub fn as_sampler(&self) -> Option<Texture>

Tries to unwrap property value as texture.