TextureData

Struct TextureData 

pub struct TextureData { /* private fields */ }
Actual texture data.

Implementations

Source

impl TextureData

pub fn load_from_memory( data: &[u8], compression: CompressionOptions, gen_mip_maps: bool, ) -> Result<Self, TextureError>

Tries to load a texture from given data in one of the following formats: PNG, BMP, TGA, JPG, DDS, GIF. Use this method if you want to load a texture from embedded data.

On-demand compression and mip-map generation

The data can be compressed if needed to improve performance on GPU side. Mip-maps can be generated as well. CAVEAT: Compression and mip-map generation won’t be taken into account in case of DDS textures, because DDS can already contain such data, you should generate mips and compress DDS textures manually using some offline tool like DirectXTexTool or similar.

Important notes

Textures loaded with this method won’t be correctly serialized! It means that if you’ll made a scene with textures loaded with this method, and then save a scene, then the engine won’t be able to restore the textures if you’ll try to load the saved scene. This is essential limitation of this method, because the engine does not know where to get the data of the texture at loading. You should use ResourceManager::request_texture in majority of cases!

Use cases

Main use cases for this method are: procedural textures, icons for GUI.

pub fn from_bytes( kind: TextureKind, pixel_kind: TexturePixelKind, bytes: Vec<u8>, serialize_content: bool, ) -> Option<Self>

Creates new texture instance from given parameters.

Limitations

Currently textures with only one mip level are supported!

pub fn set_minification_filter(&mut self, filter: TextureMinificationFilter)

Sets new minification filter. It is used when texture becomes smaller.

pub fn minification_filter(&self) -> TextureMinificationFilter

Returns current minification filter.

pub fn set_magnification_filter(&mut self, filter: TextureMagnificationFilter)

Sets new magnification filter. It is used when texture is “stretching”.

pub fn magnification_filter(&self) -> TextureMagnificationFilter

Returns current magnification filter.

pub fn set_s_wrap_mode(&mut self, s_wrap_mode: TextureWrapMode)

Sets new S coordinate wrap mode.

pub fn s_wrap_mode(&self) -> TextureWrapMode

Returns current S coordinate wrap mode.

pub fn set_t_wrap_mode(&mut self, t_wrap_mode: TextureWrapMode)

Sets new T coordinate wrap mode.

pub fn t_wrap_mode(&self) -> TextureWrapMode

Returns current T coordinate wrap mode.

pub fn mip_count(&self) -> u32

Returns total mip count.

pub fn kind(&self) -> TextureKind

Returns texture kind.

pub fn data_hash(&self) -> u64

Returns current data hash. Hash is guaranteed to be in actual state.

pub fn pixel_kind(&self) -> TexturePixelKind

Returns current pixel kind.

pub fn data(&self) -> &[u8]

Returns current data as immutable slice.

pub fn first_mip_level_data(&self) -> &[u8]

Returns data of the first mip level.

pub fn is_procedural(&self) -> bool

Returns true if the texture is procedural, false - otherwise.

Notes

Content of procedural textures is saved during serialization and they never resolved on deserialization. Resolving here means a process of getting correct texture instance by its path.

pub fn is_render_target(&self) -> bool

Returns true if the texture is used as render target.

pub fn set_anisotropy_level(&mut self, anisotropy: f32)

Max samples for anisotropic filtering. Default value is 16.0 (max). However real value passed to GPU will be clamped to maximum supported by current GPU. To disable anisotropic filtering set this to 1.0. Typical values are 2.0, 4.0, 8.0, 16.0.

pub fn anisotropy_level(&self) -> f32

Returns current anisotropy level.

pub fn set_path<P: AsRef<Path>>(&mut self, path: P)

Sets new path to source file.

pub fn save(&self) -> Result<(), TextureError>

Tries to save internal buffer into source file.

pub fn modify(&mut self) -> TextureDataRefMut<'_>

Returns a special reference holder that provides mutable access to content of the texture and automatically calculates hash of the data in its destructor.