pub struct Surface {
pub vertex_weights: Vec<VertexWeightSet>,
pub bones: Vec<Handle<Node>>,
/* private fields */
}Expand description
See module docs.
Fields
vertex_weights: Vec<VertexWeightSet>Temporal array for FBX conversion needs, it holds skinning data (weight + bone handle)
and will be used to fill actual bone indices and weight in vertices that will be
sent to GPU. The idea is very simple: GPU needs to know only indices of matrices of
bones so we can use bones array as reference to get those indices. This could be done
like so: iterate over all vertices and weight data and calculate index of node handle that
associated with vertex in bones array and store it as bone index in vertex.
bones: Vec<Handle<Node>>Array of handle to scene nodes which are used as bones.
Implementations
Sourceimpl Surface
impl Surface
Sourcepub fn new(data: Arc<Mutex<SurfaceData>>) -> Self
pub fn new(data: Arc<Mutex<SurfaceData>>) -> Self
Creates new surface instance with given data and without any texture.
Sourcepub fn material_id(&self) -> u64
pub fn material_id(&self) -> u64
Calculates material id.
Sourcepub fn data(&self) -> Arc<Mutex<SurfaceData>>
pub fn data(&self) -> Arc<Mutex<SurfaceData>>
Returns current data used by surface.
Sourcepub fn set_material(&mut self, material: Arc<Mutex<Material>>)
pub fn set_material(&mut self, material: Arc<Mutex<Material>>)
Sets new material for the surface.