#[repr(C)]pub struct StaticVertex {
pub position: Vector3<f32>,
pub tex_coord: Vector2<f32>,
pub normal: Vector3<f32>,
pub tangent: Vector4<f32>,
}Expand description
A vertex for static meshes.
Fields
position: Vector3<f32>Position of vertex in local coordinates.
tex_coord: Vector2<f32>Texture coordinates.
normal: Vector3<f32>Normal in local coordinates.
tangent: Vector4<f32>Tangent vector in local coordinates.
Implementations
Sourceimpl StaticVertex
impl StaticVertex
Sourcepub fn from_pos_uv(position: Vector3<f32>, tex_coord: Vector2<f32>) -> Self
pub fn from_pos_uv(position: Vector3<f32>, tex_coord: Vector2<f32>) -> Self
Creates new vertex from given position and texture coordinates.
Sourcepub fn from_pos_uv_normal(
position: Vector3<f32>,
tex_coord: Vector2<f32>,
normal: Vector3<f32>,
) -> Self
pub fn from_pos_uv_normal( position: Vector3<f32>, tex_coord: Vector2<f32>, normal: Vector3<f32>, ) -> Self
Creates new vertex from given position and texture coordinates.
Sourcepub fn layout() -> &'static [VertexAttributeDescriptor]
pub fn layout() -> &'static [VertexAttributeDescriptor]
Returns layout of the vertex.