LayerDefinition

Struct LayerDefinition 

pub struct LayerDefinition {
    pub material: Arc<Mutex<Material>>,
    pub mask_property_name: String,
}
Layer definition for a terrain builder.

Fields

material: Arc<Mutex<Material>>

Material of the layer.

mask_property_name: String

Name of the mask sampler in the material. It should be maskTexture if standard material shader is used.

Implementation details

It will be used in the renderer to set appropriate chunk mask to the copy of the material.