VisibilityCache

Struct VisibilityCache 

pub struct VisibilityCache { /* private fields */ }
Visibility cache stores information about objects visibility for a single frame. Allows you to quickly check if an object is visible or not.

Notes

Visibility cache stores very coarse information about object visibility, it does not include any kind of occlusion tests of whatsoever. It just a simple frustum test + level-of-detail (LOD) system.

LODs have priority over other visibility options, if a level is not active, then its every object will be hidden, not matter if the actual visibility state is visible.

Performance

The cache is based on hash map, so it is very fast and has O(1) complexity for fetching.

Implementations

Source

impl VisibilityCache

pub fn invalidate(&mut self) -> FxHashMap<Handle<Node>, bool>

Replaces internal map with empty and returns previous value. This trick is useful to reuse hash map to prevent redundant memory allocations.

pub fn update( &mut self, graph: &Graph, observer_position: Vector3<f32>, z_near: f32, z_far: f32, frustums: Option<&[&Frustum]>, )

Updates visibility cache - checks visibility for each node in given graph, also performs frustum culling if frustum set is specified.

pub fn is_visible(&self, node: Handle<Node>) -> bool

Checks whether the node is visible or not.

Complexity

Constant, O(1)