pub struct EffectInput { /* private fields */ }Expand description
Input is a “reference” to a sound source. Samples of sound source will be either passed directly to effect or will be transformed by filter if one is set.
Implementations
Sourceimpl EffectInput
impl EffectInput
Sourcepub fn direct(source: Handle<SoundSource>) -> EffectInput
pub fn direct(source: Handle<SoundSource>) -> EffectInput
Creates new effect input using specified handle of sound source.
Sourcepub fn filtered(
source: Handle<SoundSource>,
filter: Handle<InputFilter>,
) -> EffectInput
pub fn filtered( source: Handle<SoundSource>, filter: Handle<InputFilter>, ) -> EffectInput
Creates new filtered effect input using specified handles of source and filter.
Filtered inputs are suitable for emulating occlusion of sound. For example you can add filter to input and then modify its parameters in runtime: if there is no direct path from listener to sound source - make it lowpass, otherwise - allpass.