pub struct LightmapEntry {
pub texture: Option<Texture>,
pub lights: Vec<Handle<Node>>,
}Expand description
Fields
texture: Option<Texture>Lightmap texture.
TODO: Is single texture enough? There may be surfaces with huge amount of faces which may not fit into texture, because there is hardware limit on most GPUs up to 8192x8192 pixels.
lights: Vec<Handle<Node>>List of lights that were used to generate this lightmap. This list is used for masking when applying dynamic lights for surfaces with light, it prevents double lighting.