pub struct VisibilityCache { /* private fields */ }Expand description
Visibility cache stores information about objects visibility for a single frame. Allows you to quickly check if an object is visible or not.
Notes
Visibility cache stores very coarse information about object visibility, it does not include any kind of occlusion tests of whatsoever. It just a simple frustum test + level-of-detail (LOD) system.
LODs have priority over other visibility options, if a level is not active, then its every object will be hidden,
not matter if the actual visibility state is visible.
Performance
The cache is based on hash map, so it is very fast and has O(1) complexity for fetching.
Implementations
Sourceimpl VisibilityCache
impl VisibilityCache
Sourcepub fn invalidate(&mut self) -> FxHashMap<Handle<Node>, bool>
pub fn invalidate(&mut self) -> FxHashMap<Handle<Node>, bool>
Replaces internal map with empty and returns previous value. This trick is useful to reuse hash map to prevent redundant memory allocations.