pub struct BaseEffect { /* private fields */ }Expand description
Base effect for all other kinds of effects. It contains set of inputs (direct or filtered), provides some basic methods to control them.
Implementations
Sourceimpl BaseEffect
impl BaseEffect
Sourcepub fn set_gain(&mut self, gain: f32)
pub fn set_gain(&mut self, gain: f32)
Sets effect gain. It should be in (0;1) range, but larger values still fine - they can be used to achieve “overdrive” effect if needed. Basically this value defines how “loud” effect will be.
Sourcepub fn add_filter(&mut self, filter: InputFilter) -> Handle<InputFilter>
pub fn add_filter(&mut self, filter: InputFilter) -> Handle<InputFilter>
Adds new filter to effect and returns its handle. Filter handle then can be used to add input to effect (if it is filtered input).
Sourcepub fn add_input(&mut self, input: EffectInput)
pub fn add_input(&mut self, input: EffectInput)
Adds new input to effect.
Sourcepub fn filter(&self, handle: Handle<InputFilter>) -> &InputFilter
pub fn filter(&self, handle: Handle<InputFilter>) -> &InputFilter
Returns shared reference to filter.
Sourcepub fn filter_mut(&mut self, handle: Handle<InputFilter>) -> &mut InputFilter
pub fn filter_mut(&mut self, handle: Handle<InputFilter>) -> &mut InputFilter
Returns mutable reference to filter.